package dk.nsp.api.auth

import android.net.Uri
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.tryOrFailed
import net.openid.appauth.*
import kotlin.coroutines.resume
import kotlin.coroutines.suspendCoroutine


/**
 * Helper extension function to map service discovery callback in a coroutine
 * and wrapping the result
 * @receiver AuthorizationService
 * @param forDiscoveryUrl Uri
 * @return ResultWrapper<AuthorizationServiceConfiguration>
 */
suspend fun getServiceConfiguration(forDiscoveryUrl: Uri): ResultWrapper<AuthorizationServiceConfiguration> =
    suspendCoroutine { continuation ->
        AuthorizationServiceConfiguration.fetchFromIssuer(forDiscoveryUrl) { serviceConfiguration: AuthorizationServiceConfiguration?, exception: AuthorizationException? ->
            continuation.resume(
                ResultWrapper.map(
                    serviceConfiguration,
                    exception
                        ?: AuthException.UnknownError("Something went wrong when discovering services")
                )
            )
        }
    }

/**
 * Maps [AuthorizationService.TokenResponseCallback] to a coroutine and wraps result in [ResultWrapper]
 * @receiver AuthorizationService
 * @param tokenRequest TokenRequest
 * @return ResultWrapper<TokenResponse>
 */
suspend fun AuthorizationService.performTokenRequest(tokenRequest: TokenRequest): ResultWrapper<TokenResponse> =
    suspendCoroutine { continuation ->
        performTokenRequest(tokenRequest) { response: TokenResponse?, exception: AuthorizationException? ->
            continuation.resume(
                ResultWrapper.map(
                    response,
                    exception ?: AuthException.UnknownError(
                        "Something went wrong with [authService.performTokenRequest]"
                    )
                )
            )
        }
    }

/**
 * Maps [AuthState.AuthStateAction] to a coroutine and wrapes result in [ResultWrapper]
 * @receiver AuthState
 * @return ResultWrapper<String>
 */
suspend fun AuthState.performActionWithFreshTokens(authService: AuthorizationService): ResultWrapper<String> =
    suspendCoroutine { continuation ->
        tryOrFailed {
            performActionWithFreshTokens(authService) { accessToken: String?, _: String?, exception: AuthorizationException? ->
                continuation.resume(
                    ResultWrapper.map(
                        accessToken,
                        exception ?: AuthException.UnknownError(
                            "Something went wrong while fetching accessToken"
                        )
                    )
                )
            }
        }
    }
