package dk.nsp.api.auth.keyserver.models

import android.os.Parcelable
import android.util.Base64
import dk.nsp.api.util.parceler.DateTimeParceler
import dk.nsp.shared.tryAndLog
import kotlinx.parcelize.Parcelize
import kotlinx.parcelize.TypeParceler

import kotlinx.serialization.Serializable
import org.joda.time.DateTime
import java.security.Key
import javax.crypto.Cipher
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec


/**
 * @property key String the AES key
 * @property keyId String the key's id
 * @property longSecret String the longsecret working like a second password
 * @constructor
 */
@Serializable
@Parcelize
@TypeParceler<DateTime?, DateTimeParceler>
@TypeParceler<DateTime, DateTimeParceler>
data class KeyModelDTO(
    val key: String,
    val keyId: String,
    val longSecret: String
) : Parcelable

//region Crypto helpers
private const val aesCipher: String = "AES"
private const val aesCipherCbc5Padding = "$aesCipher/CBC/PKCS5Padding"

//An iv of null's / 0's /128 bit => 16 characters
private val iv: IvParameterSpec = IvParameterSpec(ByteArray(16) { 0 })

/**
 * Encrypts the given data with [KeyModelDTO.key]
 * @receiver KeyModel
 * @param data ByteArray
 * @return ByteArray?
 */
fun KeyModelDTO.encrypt(data: ByteArray): ByteArray? {
    val encryptionKey: Key = aesKey() ?: return null
    return tryAndLog {
        Cipher.getInstance(aesCipherCbc5Padding)?.let { cipher ->
            cipher.init(Cipher.ENCRYPT_MODE, encryptionKey, iv)
            cipher.doFinal(data)
        }
    }
}

/**
 * Decrypts the given content using [KeyModelDTO.key]
 * @receiver KeyModel
 * @param data ByteArray
 * @return ByteArray?
 */
fun KeyModelDTO.decrypt(data: ByteArray): ByteArray? {
    val decryptionKey: Key = aesKey() ?: return null
    return tryAndLog {
        Cipher.getInstance(aesCipherCbc5Padding)?.let { cipher ->
            cipher.init(Cipher.DECRYPT_MODE, decryptionKey, iv)
            cipher.doFinal(data)
        }
    }
}


/**
 * Unpacks the AES key for this [KeyModelDTO]
 * @receiver KeyModel
 * @return Key?
 */
private fun KeyModelDTO.aesKey(): Key? {
    //Decode the base 64 encoding
    val decodedKey = Base64.decode(key, Base64.DEFAULT) ?: return null
    return SecretKeySpec(decodedKey, aesCipher)
}
//endregion
