package dk.nsp.api.util.parceler

import android.os.Parcel
import dk.nsp.shared.util.extensions.map
import kotlinx.parcelize.Parceler

import org.joda.time.DateTime
import org.joda.time.DateTimeZone
import org.joda.time.LocalDate
import org.joda.time.LocalTime


/**
 * wraps the ability to serialize null
 * It just stores a int and reads an int before delegating to the implementor. (balanced read/write)
 * @param [T] the type we want to work with (not null)
 */
abstract class ParcelerForNullable<T> : Parceler<T?> {
    //reads 1
    override fun create(parcel: Parcel): T? {
        val isNull = parcel.readInt() == NULL_MEANING_VALUE
        return if (isNull) {
            null
        } else {
            createNotNull(parcel)
        }
    }

    // see @Parcelable.WriteFlags (unfortunately hidden)
    //writes 1
    override fun T?.write(parcel: Parcel, flags: Int) {
        val isNull = (this == null).map(NULL_MEANING_VALUE, NOT_NULL_MEANING_VALUE)
        parcel.writeInt(isNull)
        this?.writeNotNull(parcel)
    }

    abstract fun createNotNull(parcel: Parcel): T

    abstract fun T.writeNotNull(parcel: Parcel)

    companion object {
        private const val NULL_MEANING_VALUE = -1
        private const val NOT_NULL_MEANING_VALUE = 2

    }
}

/**
 * Handles timezones as efficient as possible.
 * use with
 * `@TypeParceler<DateTime, DateTimeParceler>`
 * on a [Parcelize] class to support DateTime(s)
 * Remeber to add both types of nullable / non nullable types for [Parcelize]
 */
object DateTimeParceler : ParcelerForNullable<DateTime>() {

    //2 reads
    override fun createNotNull(parcel: Parcel): DateTime {
        val timeInMs = parcel.readLong()
        val zone = parcel.readString()
        return DateTime(timeInMs, DateTimeZone.forID(zone))
    }

    //2 writes
    override fun DateTime.writeNotNull(parcel: Parcel) {
        parcel.writeLong(millis)
        parcel.writeString(zone.id)
    }

}


object LocalTimeParceler : ParcelerForNullable<LocalTime>() {
    //1 read
    override fun createNotNull(parcel: Parcel): LocalTime {
        return LocalTime.parse(parcel.readString())
    }

    // 1 write
    override fun LocalTime.writeNotNull(parcel: Parcel) {
        parcel.writeString(this.toString())
    }

}

object LocalDateParceler : ParcelerForNullable<LocalDate>() {
    //1 read
    override fun createNotNull(parcel: Parcel): LocalDate {
        return LocalDate.parse(parcel.readString())
    }

    //1 write
    override fun LocalDate.writeNotNull(parcel: Parcel) {
        parcel.writeString(this.toString())
    }

}