package dk.nsp.mingraviditet.ui.main.appointments

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.constraintlayout.motion.widget.MotionLayout
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.commonsense.android.kotlin.views.extensions.setup
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.domain.careplan.CareplanActivities
import dk.nsp.domain.careplan.CareplanActivity
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentAppointmentsBinding
import dk.nsp.mingraviditet.ui.main.appointments.rows.*
import dk.nsp.mingraviditet.ui.widgets.BottomInfoToggler
import dk.nsp.mingraviditet.util.android.motionlayout.MotionLayoutStateEnum
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.*

@AndroidEntryPoint
class AppointmentsFragment : Fragment() {
    private lateinit var binding: FragmentAppointmentsBinding

    private val adapter by lazy {
        DelegatingListAdapter()
    }

    private var state: MotionLayoutStateEnum = MotionLayoutStateEnum.Start
        set(value) {
            field = value
            (binding.root as MotionLayout).handleToolbarState(value)
        }

    private val appointmentsViewModel: AppointmentsViewModel by viewModels()

    private var infoToggler: BottomInfoToggler? = null
    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {
        binding = FragmentAppointmentsBinding.inflate(inflater, null, false)
        binding.toolbarTitle.setText(R.string.bottom_navigation_title_appointments)
        setupToolbarState()


        binding.recyclerview.setup(
            adapter,
            LinearLayoutManager(context, RecyclerView.VERTICAL, false)
        )

        appointmentsViewModel.careplanActivities.observe(viewLifecycleOwner) {
            val rows = mapCareplanActivitiesToRows(it)

            adapter.submitList(rows)
        }

        infoToggler = BottomInfoToggler(binding.bottomInfo, binding.recyclerview)

        appointmentsViewModel.careplansFinished.observeIfNotHandled(viewLifecycleOwner) {
            val indexOfPastFooter =
                adapter.currentList.indexOfFirst { row -> row is AppointmentsPastFooterRow }
            binding.recyclerview.smoothSnapToPosition(indexOfPastFooter)
        }

        return binding.root
    }

    private fun mapCareplanActivitiesToRows(it: CareplanActivities): List<RecyclerAdapterView<*>> {
        val rows = mutableListOf<RecyclerAdapterView<*>>()

        // Past section:
        rows.add(AppointmentInfoRow(isPast = true))
        it.pastActivities.forEach { activityYear ->
            rows.add(AppointmentsYearRow(activityYear.year.year.toString(), true))

            activityYear.monthActivities.forEach { activityMonth ->
                rows.add(AppointmentsMonthRow(activityMonth.month.toLongMonthFormat(), true))

                val activitiesForMonth = activityMonth.activites.map { activity ->
                    AppointmentRow(AppointmentRowBindingModel(activity) {
                        navigateTo(activity)
                    })
                }
                rows.addAll(activitiesForMonth)
            }
        }
        //If anything is previous we add the footer
        if (rows.isNotEmpty()) {
            rows.add(AppointmentsPastFooterRow())
        }
        // always add header for future appointments
        rows.add(AppointmentsHeaderRow())

        it.futureActivities.forEach { year ->
            rows.add(AppointmentsYearRow(year.year.year.toString(), false))

            year.monthActivities.forEach { month ->
                rows.add(AppointmentsMonthRow(month.month.toLongMonthFormat(), false))

                val activitiesForMonth = month.activites.map { activity ->
                    AppointmentRow(AppointmentRowBindingModel(activity) {
                        navigateTo(activity)
                    })
                }
                rows.addAll(activitiesForMonth)
            }
        }
        rows.add(AppointmentInfoRow(isPast = false))
        return rows
    }

    private fun navigateTo(activity: CareplanActivity) {
        findNavController().navigateSafe(
                AppointmentsFragmentDirections.actionKnowledgeFragmentToCareplanActivityDetailsFragment(
                        activity
                )
        )
    }

    private fun setupToolbarState() {
        setupToolbarStateListener(binding.root) { newState ->
            state = newState
        }

        (binding.root as MotionLayout).handleToolbarState(state)
    }
}