/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.appointments

import androidx.hilt.Assisted
import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.*
import com.commonsense.android.kotlin.base.extensions.collections.categorizeInto
import com.commonsense.android.kotlin.base.extensions.coroutines.launchIO
import dk.nsp.domain.careplan.*
import dk.nsp.domain.repository.CareplanRepository
import dk.nsp.mingraviditet.util.extensions.createWithYear
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import org.joda.time.DateTime

class AppointmentsViewModel @ViewModelInject constructor(
        private val careplanRepository: CareplanRepository,
        @Assisted private val savedStateHandle: SavedStateHandle
) : ViewModel() {

    private val _careplansMapped: MutableLiveData<CareplanActivities> = MutableLiveData()
    val careplanActivities: LiveData<CareplanActivities> = _careplansMapped

    val careplansFinished: LiveData<SingleEvent<Unit>> = MediatorLiveData<SingleEvent<Unit>>().apply {
        addSource(_careplansMapped) {
            postValue(SingleEvent(Unit))
        }
    }

    init {
        getCareplans()
    }

    private fun getCareplans() = viewModelScope.launchIO {
        val res = careplanRepository.getCareplans(viewModelScope).await()

        when (res) {
            is Success -> {
                sortActivitiesIntoModel(res.value.first())
            }
            is Failed -> {

            }
        }
    }

    private fun sortActivitiesIntoModel(careplan: Careplan) =
            viewModelScope.launchIO {

                // Sort all activities into past and future:
                val (past, future) = careplan.activities.categorizeInto(
                        { it.sortDate.isBeforeNow },
                        { it.sortDate.isAfterNow })

                //Make past section first
                val pastActivites = mapActivitiesIntoActivityYears(past)

                // Make future section afterwards:
                val futureActivities = mapActivitiesIntoActivityYears(future)

                //Final product:
                _careplansMapped.postValue(CareplanActivities(pastActivites, futureActivities))
            }

    private fun mapActivitiesIntoActivityYears(activities: List<CareplanActivity>): List<CareplanActivityYear> {
        val yearSet = activities.map { it.sortDate.year }.toSet()

        return yearSet.map { year ->
            val activitiesForYear = activities.filter { it.sortDate.year == year }
            val monthsOfActivityYear =
                    activitiesForYear.map { it.sortDate.monthOfYear }.toSet()

            val careplanActivitiesMonth = monthsOfActivityYear.map { month ->
                val activitiesForMonth =
                        activities.filter { it.sortDate.year == year && it.sortDate.monthOfYear == month }
                CareplanActivitiesMonth(DateTime(year, month, 1, 0, 0), activitiesForMonth)
            }

            CareplanActivityYear(DateTime().createWithYear(year), careplanActivitiesMonth)
        }
    }
}