/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.journal.extensions

import androidx.annotation.StringRes
import dk.nsp.domain.dataCard.*
import dk.nsp.mingraviditet.R
import dk.nsp.util.TextModel
import dk.nsp.util.TextResourceModel

fun YesNoEnum?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            YesNoEnum.Yes -> R.string.enum_yes
            YesNoEnum.No -> R.string.enum_no
            else -> R.string.common_missing_data
        }
    )
}

fun PositiveNegativeType?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            PositiveNegativeType.Positive -> R.string.enum_positive
            PositiveNegativeType.Negative -> R.string.enum_negative
            null -> R.string.common_missing_data
        }
    )
}

fun PositiveNegativeDeclinedType?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            PositiveNegativeDeclinedType.Positive -> R.string.enum_positive
            PositiveNegativeDeclinedType.Negative -> R.string.enum_negative
            PositiveNegativeDeclinedType.Declined -> R.string.enum_declined
            else -> R.string.common_missing_data
        }
    )
}

fun ExpectedDateOfDeliveryCertainty?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            ExpectedDateOfDeliveryCertainty.Certain -> R.string.enum_certainty_certain
            ExpectedDateOfDeliveryCertainty.Equivocal -> R.string.enum_certainty_equivocal
            null -> R.string.common_missing_data
        }
    )
}

fun AssistedFertilizationType?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            AssistedFertilizationType.Spontaneous -> R.string.assisted_fertilization_type_spontaneous
            AssistedFertilizationType.OvulationInductionIvf -> R.string.assisted_fertilization_type_ovulation_induction_ivf
            AssistedFertilizationType.Ivf -> R.string.assisted_fertilization_type_ivf
            AssistedFertilizationType.Icsi -> R.string.assisted_fertilization_type_icsi
            AssistedFertilizationType.HormoneTherapy -> R.string.assisted_fertilization_type_hormone_therapy
            AssistedFertilizationType.Insemination -> R.string.assisted_fertilization_type_insemination
            AssistedFertilizationType.EggDonation -> R.string.assisted_fertilization_type_egg_donation
            AssistedFertilizationType.Unknown -> R.string.assisted_fertilization_type_unknown
            null -> R.string.common_missing_data
        }
    )
}

fun AllergicPredispositionOfChild?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            AllergicPredispositionOfChild.NoPredisposition -> R.string.allergic_predisposition_no
            AllergicPredispositionOfChild.ParentalPredisposition -> R.string.allergic_predisposition_parental
            AllergicPredispositionOfChild.SiblingPredisposition -> R.string.allergic_predisposition_sibling
            AllergicPredispositionOfChild.DoublePredisposition -> R.string.allergic_predisposition_double
            null -> R.string.common_missing_data
        }
    )
}

fun List<ChronicConditions>?.toTextModel(): TextModel {
    val conditionsAsTextModels = this?.map {
        TextModel.from(it.toStringRes())
    }?.toTypedArray() ?: arrayOf()

    val commaSeparatedTemplate = conditionsAsTextModels.getCommaSeparatedStringResForSize()

    return TextModel.from(commaSeparatedTemplate, *conditionsAsTextModels)
}

fun ChronicConditions.toStringRes(): Int {
    return when (this) {
        ChronicConditions.CirculatorySystem -> R.string.chronic_condition_circulatory_system
        ChronicConditions.RespiratorySystem -> R.string.chronic_condition_respiratory_system
        ChronicConditions.Thyroid -> R.string.chronic_condition_thyroid
        ChronicConditions.Diabetes -> R.string.chronic_condition_diabetes
        ChronicConditions.Epilepsy -> R.string.chronic_condition_epilepsy
        ChronicConditions.MentalIllness -> R.string.chronic_condition_mental_illness
        ChronicConditions.Herpes -> R.string.chronic_condition_herpes
        ChronicConditions.Uvi -> R.string.chronic_condition_uvi
        ChronicConditions.ConeSection -> R.string.chronic_condition_cone_section
        ChronicConditions.Mrsa -> R.string.chronic_condition_mrsa
        ChronicConditions.Gbs -> R.string.chronic_condition_gbs
    }
}

private fun Array<TextResourceModel>.getCommaSeparatedStringResForSize(): Int {
    return when (this.size) {
        0 -> R.string.zero_items_comma_divided
        1 -> R.string.one_item_comma_divided
        2 -> R.string.two_items_comma_divided
        3 -> R.string.three_items_comma_divided
        4 -> R.string.four_items_comma_divided
        5 -> R.string.five_items_comma_divided
        6 -> R.string.six_items_comma_divided
        7 -> R.string.seven_items_comma_divided
        8 -> R.string.eight_items_comma_divided
        9 -> R.string.nine_items_comma_divided
        10 -> R.string.ten_items_comma_divided
        11 -> R.string.eleven_items_comma_divided
        else -> R.string.zero_items_comma_divided
    }
}

fun VaccinationStatus?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            VaccinationStatus.Vaccinated -> R.string.vaccination_status_vaccinated
            VaccinationStatus.NotVaccinated -> R.string.vaccination_status_not_vaccinated
            VaccinationStatus.Unknown -> R.string.vaccination_status_unknown
            null -> R.string.common_missing_data
        }
    )
}

fun TobaccoUser?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            TobaccoUser.Active -> R.string.tobacco_usage_active
            TobaccoUser.Never -> R.string.tobacco_usage_never
            TobaccoUser.FormerBecausePregnancy -> R.string.tobacco_usage_former_because_pregnancy
            TobaccoUser.FormerLongAgo -> R.string.tobacco_usage_former_long_ago
            null -> R.string.common_missing_data
        }
    )
}

@StringRes
fun WorkEnvironmentRiskType.toStringRes(): Int {
    return when (this) {
        WorkEnvironmentRiskType.Biological -> R.string.work_environment_risk_type_biological
        WorkEnvironmentRiskType.Chemical -> R.string.work_environment_risk_type_chemical
        WorkEnvironmentRiskType.Ergonomic -> R.string.work_environment_risk_type_ergonomic
        WorkEnvironmentRiskType.Other -> R.string.work_environment_risk_type_other
    }
}

fun List<WorkEnvironmentRiskType>?.toCommaSeparatedTextModel(): TextModel {
    val conditionsAsTextModels = this?.map {
        TextModel.from(it.toStringRes())
    }?.toTypedArray() ?: arrayOf()

    val commaSeparatedTemplate = conditionsAsTextModels.getCommaSeparatedStringResForSize()

    return TextModel.from(commaSeparatedTemplate, *conditionsAsTextModels)
}

fun PrenatalSickLeave?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            PrenatalSickLeave.None -> R.string.prenatal_sick_leave_none
            PrenatalSickLeave.Partial -> R.string.prenatal_sick_leave_partial
            PrenatalSickLeave.Full -> R.string.prenatal_sick_leave_full
            null -> R.string.common_missing_data
        }
    )
}

fun Sex?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            Sex.Female -> R.string.child_sex_female
            Sex.Male -> R.string.child_sex_male
            null -> R.string.common_missing_data
        }
    )
}

fun BirthOutcome?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            BirthOutcome.Livebirth -> R.string.child_outcome_livebirth
            BirthOutcome.Stillbirth -> R.string.child_outcome_stillbirth
            null -> R.string.common_missing_data
        }
    )
}

fun SubjectiveBirthExperience?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            SubjectiveBirthExperience.Good -> R.string.subjective_experience_good
            SubjectiveBirthExperience.Normal -> R.string.subjective_experience_normal
            SubjectiveBirthExperience.Bad -> R.string.subjective_experience_bad
            null -> R.string.common_missing_data
        }
    )
}

fun AbortionType?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            AbortionType.Spontaneous -> R.string.abortion_type_spontaneous
            AbortionType.Induced -> R.string.abortion_type_induced
            null -> R.string.common_missing_data
        }
    )
}

fun ParentalCraftClass?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            ParentalCraftClass.Individual -> R.string.midwife_consultation_form_individual
            ParentalCraftClass.Group -> R.string.midwife_consultation_form_group
            null -> R.string.common_missing_data
        }
    )
}

fun MaritalOrPartnershipStatus?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            MaritalOrPartnershipStatus.Single -> R.string.partnership_status_single
            MaritalOrPartnershipStatus.Married -> R.string.partnership_status_married
            MaritalOrPartnershipStatus.Separated -> R.string.partnership_status_separated
            MaritalOrPartnershipStatus.Divorced -> R.string.partnership_status_divorced
            MaritalOrPartnershipStatus.Widowed -> R.string.partnership_status_widowed
            null -> R.string.common_missing_data
        }
    )
}

fun LivingConditions?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            LivingConditions.Partnership -> R.string.living_conditions_partnership
            LivingConditions.Single -> R.string.living_conditions_single
            null -> R.string.common_missing_data
        }
    )
}

fun CareLevel?.toTextModel(): TextModel {
    return TextModel.from(
        when (this) {
            CareLevel.Basic -> R.string.prenatal_carelevel_basic
            CareLevel.Extended -> R.string.prenatal_carelevel_extended
            CareLevel.SpecialistMultidisciplinaryTeam -> R.string.prenatal_carelevel_specialist_multidisciplinary_team
            CareLevel.HighRiskPregnancyCare -> R.string.prenatal_carelevel_high_rish_pregnancy_care
            null -> R.string.common_missing_data
        }
    )
}
