/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.journal.sections.measurements

import androidx.hilt.Assisted
import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dk.nsp.domain.dataCard.MaternityDataCard
import dk.nsp.domain.dictionary.DictionaryEntry
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.domain.repository.DictionaryRepository
import dk.nsp.domain.repository.ObservationsRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.adapterviews.KeyValueData
import dk.nsp.mingraviditet.ui.adapterviews.KeyValueRow
import dk.nsp.mingraviditet.ui.adapterviews.KeyValueSectionHeader
import dk.nsp.mingraviditet.ui.main.journal.extensions.textModel
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.getValue
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.extensions.transform
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.util.launchDefault
import dk.nsp.util.TextModel
import kotlinx.coroutines.CoroutineScope
import timber.log.Timber

class JournalMeasurementsViewModel @ViewModelInject constructor(
    private val dataCardRepository: DataCardRepository,
    private val dictionaryRepository: DictionaryRepository,
    private val observationsRepository: ObservationsRepository,
    @Assisted private val savedStateHandle: SavedStateHandle
) : ViewModel() {

    val observations = observationsRepository.observations.transform {
        it
    }

    val bodyRows = dataCardRepository.currentDataCard.transform {
        getBodyMeasurementRows(it)
    }

    private val _onDictionary = MutableLiveData<SingleEvent<DictionaryEntry>>()
    val onDictionary by _onDictionary

    init {
        getObservationsFromRepository()
    }

    private fun getObservationsFromRepository() {
        viewModelScope.launchDefault {
            val observationsRes =
                observationsRepository.getObservationsAsync(viewModelScope).await()

            when (observationsRes) {
                is Success -> {
                    Timber.d("Observations size: ${observationsRes.value.size}")
                }
                is Failed -> {
                    Timber.e(observationsRes.error)
                }
            }
        }
    }

    private fun dictionaryCallback(key: TextModel) =
        getDictionaryEntryFor(viewModelScope, key)?.let {
            { _onDictionary.postValue(it.toSingleEvent()) }
        }

    private fun getDictionaryEntryFor(scope: CoroutineScope, key: TextModel) =
        dictionaryRepository.getDictionaryEntryFor(scope, key)


    private fun getBodyMeasurementRows(maternityDataCard: MaternityDataCard): List<RecyclerAdapterView<*>> {
        val bodySection = maternityDataCard.bodyMeasuresSection

        val headerRow = KeyValueSectionHeader(
            TextModel.from(R.string.journal_section_measurements_weight_before_maternity_title)
        )

        val weightRow = KeyValueRow(
            KeyValueData(
                TextModel.from(R.string.journal_section_measurements_weight_before_maternity_value_title),
                bodySection?.prenatalBodyWeight.textModel(),
                null,
                null
            )
        )

        val heightRow = KeyValueRow(
            KeyValueData(
                TextModel.from(R.string.journal_section_measurements_height_before_maternity_value_title),
                bodySection?.bodyHeight.textModel(),
                null,
                null
            )
        )

        val title =
            TextModel.from(R.string.journal_section_measurements_bmi_before_maternity_value_title)
        val bmiRow = KeyValueRow(
            KeyValueData(
                title,
                bodySection?.bodyMassIndex.textModel(),
                null,
                dictionaryCallback(title)
            )
        )

        return listOf(headerRow, weightRow, heightRow, bmiRow)
    }
}