/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.journal.sections.scans

import androidx.hilt.Assisted
import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.*
import com.commonsense.android.kotlin.base.extensions.collections.categorize
import dk.nsp.domain.careplan.Observation
import dk.nsp.domain.dataCard.MaternityDataCard
import dk.nsp.domain.dictionary.DictionaryEntry
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.domain.repository.DictionaryRepository
import dk.nsp.domain.repository.ObservationsRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.adapterviews.KeyValueData
import dk.nsp.mingraviditet.ui.adapterviews.KeyValueRow
import dk.nsp.mingraviditet.ui.adapterviews.KeyValueSectionHeader
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.toMediumFormat
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.extensions.toTextModelOrDefault
import dk.nsp.mingraviditet.util.extensions.transform
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.shared.Failed
import dk.nsp.shared.Success
import dk.nsp.shared.util.launchDefault
import dk.nsp.util.TextModel
import kotlinx.coroutines.CoroutineScope
import timber.log.Timber

class JournalScansViewModel @ViewModelInject constructor(
    private val dataCardRepository: DataCardRepository,
    private val dictionaryRepository: DictionaryRepository,
    private val observationsRepository: ObservationsRepository,
    @Assisted private val savedStateHandle: SavedStateHandle
) : ViewModel() {

    private val _rowsToShow: MutableLiveData<List<RecyclerAdapterView<*>>> = MutableLiveData()
    val rowsToShow: LiveData<List<RecyclerAdapterView<*>>> = _rowsToShow

    private val _onDictionary: MutableLiveData<SingleEvent<DictionaryEntry>> = MutableLiveData()
    val onDictionary: LiveData<SingleEvent<DictionaryEntry>> = _onDictionary

    val testRows = dataCardRepository.currentDataCard.transform {
        createKeyValueRows(it)
    }


    val observations = observationsRepository.observations.transform {
        createTableRows(it)
        it
    }

    init {
        getObservationsFromRepository()
    }

    private fun getObservationsFromRepository() {
        viewModelScope.launchDefault {
            val observationsRes =
                observationsRepository.getObservationsAsync(viewModelScope).await()

            when (observationsRes) {
                is Success -> {
                    Timber.d("Observations size: ${observationsRes.value.size}")
                }
                is Failed -> {
                    Timber.e(observationsRes.error)
                }
            }
        }
    }

    //TODO
    private fun createTableRows(observations: List<Observation>) {
        val mapOfObservations = observations.categorize { it.fetus ?: "Ukendt" }

        Timber.d("Observations: ${mapOfObservations.size}")

    }

    private fun createKeyValueRows(dataCard: MaternityDataCard): List<RecyclerAdapterView<*>> {

        val expectedDateOfDeliveryFromUltrasoundScanKey =
            TextModel.from(R.string.journal_section_scans_expected_date_of_delivery_from_prenatal_ultrasound_scan)
        val oddsOfDownsKey = TextModel.from(R.string.journal_section_scans_downs_probability)
        val multipleRiskFactorsKey =
            TextModel.from(R.string.journal_section_scans_multiple_risk_factors)
        val multipleRiskFactorsCommentKey =
            TextModel.from(R.string.journal_section_scans_multiple_risk_factors_remarks)
        val amountOfFetusesKey = TextModel.from(R.string.journal_section_scans_amount_of_fetuses)
        val placentalBiopsyObservationKey =
            TextModel.from(R.string.journal_section_scans_placental_biopsy_observation)
        val prenatalAmniocentisisObservationKey =
            TextModel.from(R.string.journal_section_scans_prenatal_amniocentesis_observation)

        return listOf(
            KeyValueSectionHeader(TextModel.from(R.string.journal_section_scans_other_results_title)),
            KeyValueRow(
                KeyValueData(
                    expectedDateOfDeliveryFromUltrasoundScanKey,
                    dataCard.expectedDateOfDeliverySection?.expectedDateOfDeliveryFromPrenatalUltrasoundScan.toMediumFormat()
                        .toTextModelOrDefault(),
                    null,
                    dictionaryCallback(expectedDateOfDeliveryFromUltrasoundScanKey)
                )
            ),
            KeyValueRow(
                KeyValueData(
                    oddsOfDownsKey,
                    dataCard.prenatalRiskAssessmentSection?.finalRiskAssessment?.downsScreeningProbabilityAssessment.toTextModelOrDefault(),
                    null,
                    dictionaryCallback(oddsOfDownsKey)
                )
            ),
            // TODO Get data from the right place
            KeyValueRow(
                KeyValueData(
                    multipleRiskFactorsKey,
                    TextModel.from("TODO - Hvor hentes dette fra?"),
                    null,
                    dictionaryCallback(multipleRiskFactorsKey)
                )
            ),
            // TODO Get data from the right place
            KeyValueRow(
                KeyValueData(
                    multipleRiskFactorsCommentKey,
                    TextModel.from("TODO - Hvor hentes dette fra?"), //dataCard.overallAssessmentSection?.prenatalCareAssessment?.comment.toTextModelOrDefault(),
                    null,
                    dictionaryCallback(multipleRiskFactorsCommentKey)
                )
            ),
            KeyValueRow(
                KeyValueData(
                    amountOfFetusesKey,
                    dataCard.overallAssessmentSection?.prenatalCareAssessment?.numberOfFetuses?.toString()
                        .toTextModelOrDefault(),
                    null,
                    dictionaryCallback(amountOfFetusesKey)
                )
            ),
            KeyValueRow(
                KeyValueData(
                    placentalBiopsyObservationKey,
                    dataCard.prenatalRiskAssessmentSection?.finalRiskAssessment?.placentalBiopsiObservation?.result.toTextModelOrDefault(),
                    dataCard.prenatalRiskAssessmentSection?.finalRiskAssessment?.placentalBiopsiObservation?.date,
                    dictionaryCallback(placentalBiopsyObservationKey)
                )
            ),
            KeyValueRow(
                KeyValueData(
                    prenatalAmniocentisisObservationKey,
                    dataCard.prenatalRiskAssessmentSection?.finalRiskAssessment?.prenatalAmniocentesisiObservation?.result.toTextModelOrDefault(),
                    dataCard.prenatalRiskAssessmentSection?.finalRiskAssessment?.prenatalAmniocentesisiObservation?.date,
                    dictionaryCallback(prenatalAmniocentisisObservationKey)
                )
            )
        )
    }

    private fun dictionaryCallback(key: TextModel): (() -> Unit)? {
        return getDictionaryEntryFor(
            viewModelScope,
            key
        )?.let { { _onDictionary.postValue(it.toSingleEvent()) } }
    }

    private fun getDictionaryEntryFor(scope: CoroutineScope, key: TextModel): DictionaryEntry? {
        return dictionaryRepository.getDictionaryEntryFor(scope, key)
    }
}