/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.journal.sections.sectiondetails.aroundthepregnancy

import dk.nsp.domain.dataCard.*
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.util.extensions.toMediumFormat
import dk.nsp.mingraviditet.util.extensions.toTextModelOrDefault
import dk.nsp.util.TextModel
import dk.nsp.util.toTextModel
import org.joda.time.Days

fun ExpectedDateOfDeliveryFromLastPeriod?.lastFirstDayOfCycleToTextModel(): TextModel =
    this?.lastPeriod.toMediumFormat()?.let { TextModel.from(it) }
        ?: TextModel.from(R.string.common_missing_data)

fun Duration?.toTextModel(): TextModel {
    return when (this) {
        is Duration.DurationInDays -> TextModel.from("$value $unit")
        is Duration.DurationFromDates -> {
            val calc = this.calculated?.let { TextModel.from("${it.value} ${it.unit}") }
            if (calc != null) {
                calc
            } else {
                val start = this.start ?: return TextModel.from(R.string.common_missing_data)
                val end = this.end ?: return TextModel.from(R.string.common_missing_data)

                TextModel.from(Days.daysBetween(start, end).toString())
            }
        }
        null -> TextModel.from(R.string.common_missing_data)
    }
}

fun ExpectedDateOfDeliveryFromLastPeriod?.getCalculatedExpectedDeliveryDate(): TextModel {
    return this?.expectedDateOfDelivery.toMediumFormat()?.let { TextModel.from(it) }
        ?: TextModel.from(R.string.common_missing_data)
}

fun WeeklyAlcoholUnitsConsumption?.getConsumption(): TextModel {
    val value = TextModel.from("${this?.value ?: 0}")
    val unit = this?.unit?.let { TextModel.from(it) }

    return if (unit == null) {
        TextModel.from(R.plurals.alcohol_units, this?.value ?: 0, value)
    } else {
        TextModel.from(R.string.space_between_values, value, unit)
    }
}

fun WeeklyConsumptionDuringPregnancy?.getAverageConsumption(): TextModel {
    val value = TextModel.from("${this?.value ?: 0}")
    val unit = this?.unit?.let { TextModel.from(it) }

    return if (unit == null) {
        TextModel.from(R.plurals.alcohol_units, this?.value ?: 0, value)
    } else {
        TextModel.from(R.string.space_between_values, value, unit)
    }
}

fun WeeklyConsumptionDuringPregnancy?.getExcessiveDrinkingCount(): TextModel {
    val value = TextModel.from("${this?.excessiveDrinkingCount ?: 0}")

    return TextModel.from(R.plurals.excessive_drinking, this?.value ?: 0, value)
}

fun WeeklyConsumptionDuringPregnancy?.getExcessiveDrinkingWeeks(): TextModel {
    val excessiveDrinkingWeeksCommaSeparated =
        this?.excessiveDrinkingWeeks?.joinToString { it.toString() }

    return excessiveDrinkingWeeksCommaSeparated.toTextModelOrDefault()
}

fun WeeklyConsumptionDuringPregnancy?.getTweakPoints(): TextModel {
    return this?.tweakPoints?.ordinal?.let { TextModel.from(it.toString()) }
        ?: TextModel.from(R.string.common_missing_data)
}

fun ExposureToWorkEnvironmentRisk?.getExposureDuration(): TextModel {
    val start = this?.start?.toMediumFormat()?.toTextModel()
    val end = this?.end?.toMediumFormat()?.toTextModel()

    return if (start != null && end != null) {
        TextModel.from(R.string.dash_separated_strings, start, end)
    } else if (start != null && end == null) {
        TextModel.from(R.string.dash_separated_strings, start, "".toTextModel())
    } else if (start == null && end != null) {
        TextModel.from(R.string.dash_separated_strings, "".toTextModel(), end)
    } else {
        TextModel.from(R.string.common_missing_data)
    }
}