/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.mingraviditet.ui.main.journal.sections.sectiondetails.aroundthepregnancy.previouspregnancies

import androidx.hilt.Assisted
import androidx.hilt.lifecycle.ViewModelInject
import androidx.lifecycle.*
import dk.nsp.domain.dataCard.Birth
import dk.nsp.domain.dataCard.Child
import dk.nsp.domain.dictionary.DictionaryEntry
import dk.nsp.domain.repository.DataCardRepository
import dk.nsp.domain.repository.DictionaryRepository
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.ui.adapterviews.*
import dk.nsp.mingraviditet.ui.main.journal.extensions.toTextModel
import dk.nsp.mingraviditet.ui.main.journal.sections.sectiondetails.aroundthepregnancy.toTextModel
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.mingraviditet.util.extensions.toSingleEvent
import dk.nsp.mingraviditet.util.extensions.toTextModelOrDefault
import dk.nsp.mingraviditet.util.extensions.transform
import dk.nsp.mingraviditet.util.livedata.SingleEvent
import dk.nsp.util.TextModel
import kotlinx.coroutines.CoroutineScope

class PreviousBirthsViewModel @ViewModelInject constructor(
    dataCardRepository: DataCardRepository,
    private val dictionaryRepository: DictionaryRepository,
    @Assisted private val savedStateHandle: SavedStateHandle
) : ViewModel() {

    val previousBirths = dataCardRepository.currentDataCard.transform {
        val births = it.previousBirthsSection?.births ?: emptyList()
        if (births.isNotEmpty()) {
            createRows(births)
        } else {
            createEmptyState()
        }
    }

    private val _onDictionaryClicked: MutableLiveData<SingleEvent<DictionaryEntry>> =
        MutableLiveData()
    val onDictionaryClicked: LiveData<SingleEvent<DictionaryEntry>> = _onDictionaryClicked

    private fun createRows(previousBirths: List<Birth>): List<RecyclerAdapterView<*>> {
        return previousBirths.map {
            createRowsForBirth(it)
        }.flatten()
    }

    private fun createRowsForBirth(birth: Birth): List<RecyclerAdapterView<*>> {
        val birthSection = mutableListOf<RecyclerAdapterView<*>>()

        val childSections = birth.children?.mapIndexed { index, child ->
            createRowsForChild(index, birth, child)
        }?.flatten()

        childSections?.let(birthSection::addAll)

        val gestationLengthKey =
            TextModel.from(R.string.journal_section_previous_births_child_gestation_length)

        val birthRows = listOf(
            KeyValueRow(
                KeyValueData(
                    TextModel.from(R.string.journal_section_previous_births_child_year),
                    birth.date?.year?.toString().toTextModelOrDefault(),
                    null,
                    null
                )
            ),
            KeyValueRow(
                KeyValueData(
                    gestationLengthKey,
                    birth.gestationLength.toTextModel(),
                    null,
                    getDictionaryCallback(gestationLengthKey)
                )
            ),
            KeyValueRow(
                KeyValueData(
                    TextModel.from(R.string.journal_section_previous_births_child_place_of_birth),
                    birth.birthPlace.toTextModelOrDefault(),
                    null,
                    null
                )
            ),
            KeyValueRow(
                KeyValueData(
                    TextModel.from(R.string.journal_section_previous_births_child_subjective_experience),
                    birth.subjectiveBirthExperience.toTextModel(),
                    null,
                    null
                )
            ),
            KeyValueRow(
                KeyValueData(
                    TextModel.from(R.string.journal_section_previous_births_pregnancy_summary),
                    birth.pregnancySummary.toTextModelOrDefault(),
                    null,
                    null
                )
            ),
            KeyValueRow(
                KeyValueData(
                    TextModel.from(R.string.journal_section_previous_births_birth_summary),
                    birth.birthSummary.toTextModelOrDefault(),
                    null,
                    null
                )
            )
        )

        birthSection.addAll(birthRows)

        return birthSection
    }

    private fun createRowsForChild(
        index: Int,
        birth: Birth,
        child: Child
    ): List<RecyclerAdapterView<*>> {
        return listOf<RecyclerAdapterView<*>>(
            TableHeaderRow(
                TextModel.from(
                    R.string.journal_section_previous_births_child_title,
                    TextModel.from(if (birth.children?.size == 1) "" else (index + 1).toString())
                )
            ),
            TableRow(
                TableRowBindingModel(
                    TextModel.from(R.string.journal_section_previous_births_child_sex),
                    child.sex.toTextModel(),
                    0,
                    null
                )
            ),
            TableRow(
                TableRowBindingModel(
                    TextModel.from(R.string.journal_section_previous_births_child_weight),
                    child.birthWeight.toTextModel(),
                    1,
                    null,
                )
            ),
            TableRow(
                TableRowBindingModel(
                    TextModel.from(R.string.journal_section_previous_births_child_outcome),
                    child.outcome.toTextModel(),
                    2,
                    null
                )
            ),
            TableRow(
                TableRowBindingModel(
                    TextModel.from(R.string.journal_section_previous_births_child_current_health),
                    child.childsCurrentHealthStatus.toTextModelOrDefault(),
                    3,
                    null
                )
            )
        )
    }

    private fun getDictionaryCallback(forKey: TextModel): (() -> Unit)? {
        return getDictionaryEntryFor(
            viewModelScope,
            forKey
        )?.let { { _onDictionaryClicked.postValue(it.toSingleEvent()) } }
    }

    private fun createEmptyState(): List<RecyclerAdapterView<*>> {
        return listOf(
            EmptyStateFullHeight(
                TextModel.from(R.string.journal_section_previous_births_empty_state_title),
                TextModel.from(R.string.journal_section_previous_births_empty_state_description)
            )
        )
    }

    private fun getDictionaryEntryFor(scope: CoroutineScope, key: TextModel): DictionaryEntry? {
        return dictionaryRepository.getDictionaryEntryFor(scope, key)
    }
}