package dk.nsp.mingraviditet.ui.main.knowledge

import android.app.Activity
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowInsets
import android.view.inputmethod.InputMethodManager
import androidx.core.widget.doOnTextChanged
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import com.commonsense.android.kotlin.views.extensions.scrollToTop
import com.google.android.material.snackbar.Snackbar
import dagger.hilt.android.AndroidEntryPoint
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.FragmentKnowledgeBinding
import dk.nsp.mingraviditet.util.android.recycler.DelegatingListAdapter
import dk.nsp.mingraviditet.util.extensions.handleToolbarState
import dk.nsp.mingraviditet.util.extensions.navigateSafe
import dk.nsp.mingraviditet.util.extensions.observeIfNotHandled
import dk.nsp.mingraviditet.util.extensions.setupToolbarStateListener

@AndroidEntryPoint
class KnowledgeFragment : Fragment() {
    private lateinit var binding: FragmentKnowledgeBinding

    private val knowledgeViewModel: KnowledgeViewModel by viewModels()

    private val adapter by lazy {
        DelegatingListAdapter()
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentKnowledgeBinding.inflate(inflater, container, false)
        binding.toolbarTitle.setText(R.string.bottom_navigation_title_knowledge)

        binding.recyclerview.adapter = adapter

        knowledgeViewModel.onChecklistClicked.observeIfNotHandled(viewLifecycleOwner) {
            findNavController().navigateSafe(
                KnowledgeFragmentDirections.actionKnowledgeFragmentToChecklistDetailsFragment(it.id)
            )
        }

        knowledgeViewModel.onDataCombined.observe(viewLifecycleOwner) {
            adapter.submitList(it) {
                binding.recyclerview.scrollToTop(false)
                binding.recyclerview.scheduleLayoutAnimation()
            }
        }

        knowledgeViewModel.onError.observeIfNotHandled(viewLifecycleOwner) {
            Snackbar.make(binding.root, it, Snackbar.LENGTH_LONG)
                .setAction(R.string.common_ok) {}
                .show()
        }

        knowledgeViewModel.onArticleCategoryClicked.observeIfNotHandled(viewLifecycleOwner) {
            findNavController().navigateSafe(
                KnowledgeFragmentDirections.actionKnowledgeFragmentToKnowledgeArticlesFragment(
                    it
                )
            )
        }

        binding.searchBar.doOnTextChanged { text, _, _, _ ->
            text?.toString()?.let { searchTerm ->
                knowledgeViewModel.searchFor(searchTerm)
            }
        }

        knowledgeViewModel.searchResults.observe(viewLifecycleOwner) {
            adapter.submitList(it)
        }

        knowledgeViewModel.onSearchArticleClicked.observeIfNotHandled(viewLifecycleOwner) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                binding.root.windowInsetsController?.hide(WindowInsets.Type.ime())
            } else {
                val inputMethodManager =
                    context?.getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
                inputMethodManager.hideSoftInputFromWindow(binding.root.windowToken, 0)
            }
            findNavController().navigateSafe(
                KnowledgeFragmentDirections.actionKnowledgeFragmentToKnowledgeArticleFragment(
                    it.first,
                    it.second
                )
            )
        }

        setupToolbarState()

        return binding.root
    }

    private fun setupToolbarState() {
        setupToolbarStateListener(binding.motionLayout) { newState ->
            knowledgeViewModel.toolbarState = newState
            binding.motionLayout.handleToolbarState(newState)
        }

        binding.motionLayout.handleToolbarState(knowledgeViewModel.toolbarState)
    }
}