package dk.nsp.mingraviditet.ui.main.knowledge.article

import android.annotation.SuppressLint
import android.content.Context
import android.content.res.AssetManager
import android.content.res.Configuration
import android.graphics.drawable.TransitionDrawable
import android.os.Build
import android.view.MotionEvent
import android.webkit.WebSettings
import androidx.webkit.WebSettingsCompat
import androidx.webkit.WebViewFeature
import com.commonsense.android.kotlin.views.extensions.visibleOrGone
import dk.nsp.domain.knowledge.ArticleContentSection
import dk.nsp.mingraviditet.R
import dk.nsp.mingraviditet.databinding.ArticleParagraphCardRowBinding
import dk.nsp.mingraviditet.util.android.OpenExternalLinksInBrowserWebViewClient
import dk.nsp.mingraviditet.util.android.recycler.BindingViewHolder
import dk.nsp.mingraviditet.util.android.recycler.DelegationListAdapterInflater
import dk.nsp.mingraviditet.util.android.recycler.RecyclerAdapterView
import dk.nsp.shared.util.launchDefault
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import timber.log.Timber
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStreamReader


class ArticleParagraphCardRow(
    val articleContentSection: ArticleContentSection,
    val relevantId: Int
) :
    RecyclerAdapterView<ArticleParagraphCardRowBinding> {

    private var expanded = false

    @SuppressLint("ClickableViewAccessibility")
    override fun bindData(viewHolder: BindingViewHolder<ArticleParagraphCardRowBinding>) {
        viewHolder.binding.articleContent = articleContentSection

        viewHolder.binding.webview.webViewClient =
            OpenExternalLinksInBrowserWebViewClient(viewHolder.binding.root.context)

        viewHolder.binding.webview.apply {

            val htmlToLoad = createHtmlWithStyleSheet(viewHolder)
            loadDataWithBaseURL("file:///android_asset/", htmlToLoad, "text/html", "UTF-8", "")

            // Support dark mode :)
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                val nightModeFlag =
                    resources.configuration.uiMode and Configuration.UI_MODE_NIGHT_MASK

                when (nightModeFlag) {
                    Configuration.UI_MODE_NIGHT_YES -> {
                        settings.forceDark = WebSettings.FORCE_DARK_ON
                    }
                    Configuration.UI_MODE_NIGHT_NO -> {
                        settings.forceDark = WebSettings.FORCE_DARK_OFF
                    }
                    else -> {
                        settings.forceDark = WebSettings.FORCE_DARK_AUTO
                    }
                }

                if (nightModeFlag == Configuration.UI_MODE_NIGHT_YES) {
                    if (WebViewFeature.isFeatureSupported(WebViewFeature.FORCE_DARK_STRATEGY)) {
                        WebSettingsCompat.setForceDarkStrategy(
                            settings,
                            WebSettingsCompat.DARK_STRATEGY_WEB_THEME_DARKENING_ONLY // Only use the injected CSS from the HTML
                        )
                    }
                }
            }

            // Disable scroll on touch
            setOnTouchListener { _, event -> (event?.action == MotionEvent.ACTION_MOVE); }
        }

        if (relevantId == articleContentSection.id) {
            val transition = viewHolder.binding.rowBackground.background as TransitionDrawable

            CoroutineScope(Dispatchers.Main).launchDefault {
                transition.startTransition(250)
                delay(500)
                transition.reverseTransition(500)
            }

            toggleExpandCollapseState(viewHolder)
        }

        viewHolder.binding.expandCollapseButton.setOnClickListener {
            toggleExpandCollapseState(viewHolder)
        }
    }

    private fun createHtmlWithStyleSheet(viewHolder: BindingViewHolder<ArticleParagraphCardRowBinding>): String {
        val htmlToShow = readAssetHeaderAndStyleHtml(viewHolder.binding.root.context)
        val newHtml = htmlToShow.replace("[CONTENT]", articleContentSection.content)
        return newHtml
    }

    private fun readAssetHeaderAndStyleHtml(context: Context): String {
        try {
            val inputStream =
                context.assets.open("article_with_styles.html", AssetManager.ACCESS_BUFFER)
            inputStream.use { `is` ->
                val reader = BufferedReader(InputStreamReader(`is`))
                val result = StringBuilder()
                var mLine: String? = reader.readLine()
                while (mLine != null) {
                    result.append(mLine)
                    mLine = reader.readLine()
                }
                return result.toString()
            }
        } catch (e: IOException) {
            Timber.e(e, "Could not load html and style asset")
        }

        return ""
    }

    private fun toggleExpandCollapseState(viewHolder: BindingViewHolder<ArticleParagraphCardRowBinding>) {
        expanded = !expanded

        viewHolder.binding.webview.visibleOrGone(expanded)
        viewHolder.binding.contentBody.visibleOrGone(!expanded)

        if (expanded) {
            viewHolder.binding.expandCollapseButton.setText(R.string.common_cta_collapse)
        } else {
            viewHolder.binding.expandCollapseButton.setText(R.string.common_cta_expand)
        }
    }

    override val createHolderFunction: DelegationListAdapterInflater<ArticleParagraphCardRowBinding> =
        ArticleParagraphCardRowBinding::inflate
}