package dk.nsp.mingraviditet.util.commonsense_helpers

import kotlin.reflect.KProperty


/**
 * A mutable container that locks the amount of updates that are allowed
 * @param T the type of value we are to contain
 * @property maxUpdateCount [Int] the amount of updates that are allowed
 * @property value T the starting value that "may" be updated up to the provided update count
 */
public open class LockableValue<T>(
    @IntLimit(from = 1) maxUpdateCount: Int,
    initialValue: T,
    private val onUpdateRejected: (() -> Unit)? = null
) {
    @IntLimit(from = 0)
    private var remainingUpdateCount = maxUpdateCount

    /**
     * The current value (getter)
     * setting the value might not update it if the maxUpdateCount reaches zero or negative
     */
    public var value: T = initialValue
        set(newValue) {
            if (remainingUpdateCount < 0 || remainingUpdateCount == 0) {
                onUpdateRejected?.invoke()
                return
            }
            remainingUpdateCount -= 1
            field = newValue
        }
}

/**
 *
 * @receiver LockableValue<T>
 * @param any Any?
 * @param property KProperty<*>
 * @return T
 */
public operator fun <T> LockableValue<T>.getValue(any: Any?, property: KProperty<*>): T {
    return value
}

/**
 *
 * @receiver LockableValue<T>
 * @param item Any?
 * @param property KProperty<*>
 * @param newValue T
 */
public operator fun <T> LockableValue<T>.setValue(item: Any?, property: KProperty<*>, newValue: T) {
    value = newValue
}
