package dk.nsp.domain.auth


import android.app.Activity
import android.content.Context
import android.content.Intent
import androidx.fragment.app.Fragment
import dk.nsp.api.auth.AuthController
import dk.nsp.api.auth.OptionalKeyModelWithPotentialError
import dk.nsp.shared.Failed
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.Success
import dk.nsp.shared.mapSuccess
import dk.nsp.shared.util.asyncDefault
import dk.nsp.shared.util.launchDefault
import dk.nsp.shared.util.withContextMain
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred
import kotlinx.coroutines.Job
import javax.crypto.Cipher

class AuthRepo(context: Context) {
    private val authController = AuthController.get(context.applicationContext)

    fun haveLogin() = authController.hasLogin()

    fun clear() {
        logout()
        authController.clear()
    }

    fun logout() {
        authController.logout()
    }

    fun attemptToRegister(
        scope: CoroutineScope,
        callerFragment: Fragment,
        onFailed: (error: RegistrationError) -> Unit
    ): Job = scope.launchDefault {
        val intentResult = authController.getRegisterIntent()
        withContextMain {
            when (intentResult) {
                is Success -> {
                    callerFragment.startActivityForResult(
                        intentResult.value,
                        authRequestCode
                    )
                }
                is Failed -> onFailed(intentResult.error.toRegistrationError())
            }
        }
    }

    fun isResultAndOkayForRegistration(resultCode: Int, requestCode: Int): Boolean {
        return resultCode == Activity.RESULT_OK && requestCode == authRequestCode
    }

    fun handleLoginResultAndGetAccessToken(
        scope: CoroutineScope,
        data: Intent?,
        onAccessToken: () -> Unit,
        onFailed: (error: RegistrationError) -> Unit
    ): Job = scope.launchDefault {
        val loginResult = authController.handleLoginResult(data)
        when (loginResult) {
            is Failed -> onFailed(loginResult.error.toRegistrationError())
            is Success -> {
                // Login was a success, we should now be able to get an accessToken
                val accessToken = authController.getAccessToken()
                when (accessToken) {
                    is Failed -> onFailed(accessToken.error.toRegistrationError())
                    is Success -> {
                        onAccessToken()
                    }
                }
            }
        }
    }

    fun tryRestoreFromPassword(
        pinCode: String,
        scope: CoroutineScope
    ): Deferred<OptionalKeyModelWithPotentialErrorLogic> = scope.asyncDefault {
        authController.tryRestoreFromPassword(pinCode).toLogic()
    }

    fun loginWithNewPin(
        pinCode: String,
        scope: CoroutineScope
    ): Deferred<ResultWrapper<KeyModel>> = scope.asyncDefault {
        authController.storeRefreshTokenAndKeyId(pinCode).mapSuccess { it.toLogic() }
    }

    fun tryRestoreFromBiometric(
        crypto: Cipher,
        scope: CoroutineScope
    ) = scope.asyncDefault {
        authController.tryRestoreFromBiometric(crypto).toLogic()
    }

    fun storeLongSecretViaBiometric(
        cipher: Cipher,
        keyModel: KeyModel
    ): ResultWrapper<Unit> =
        authController.storeLongSecretViaBiometric(cipher, keyModel.toDTO())

    fun removeLongSecretStoredViaBiometric() {
        authController.removeLongSecretStoredViaBiometric()
    }

    fun isLoggedIn(): Boolean =
        authController.isLoggedIn()

    fun clearSession() {
        authController.clearSession()
    }

    companion object {
        const val authRequestCode = 42295
    }
}


class OptionalKeyModelWithPotentialErrorLogic(
    val keyModelResult: ResultWrapper<KeyModel>,
    val optionalKeyModel: KeyModel? = null
)

fun OptionalKeyModelWithPotentialError.toLogic(): OptionalKeyModelWithPotentialErrorLogic {

    return OptionalKeyModelWithPotentialErrorLogic(
        this.keyModelResult.mapSuccess { it.toLogic() },
        optionalKeyModel?.toLogic()
    )
}

fun OptionalKeyModelWithPotentialErrorLogic.toDTO(): OptionalKeyModelWithPotentialError {
    return OptionalKeyModelWithPotentialError(
        this.keyModelResult.mapSuccess { it.toDTO() },
        optionalKeyModel?.toDTO()
    )
}