package dk.nsp.domain.auth

import android.content.Context
import dk.nsp.api.auth.keystore.KeyStoreController
import javax.crypto.Cipher
import javax.crypto.SecretKey
import javax.crypto.spec.IvParameterSpec

class BiometricCipherKeyStore(context: Context) {

    private val aliasId = "alias"
    private val keyStoreController =
        KeyStoreController(context.applicationContext)

    /**
     * Gets a keystore cipher that is ready for the given operation.
     * @param shouldEncrypt Boolean true if we want to encrypt data, false means decrypt data
     * @return Cipher?
     */
    @Throws(Exception::class)
    fun getKeyStoreCipher(shouldEncrypt: Boolean): Cipher? {
        val (keystoreCipher, key) = getKeyStoreCipherBeforeInit() ?: return null
        if (shouldEncrypt) {
            keystoreCipher.init(Cipher.ENCRYPT_MODE, key)
        } else {
            val iv = keyStoreController.getLongSecretIv() ?: return null
            keystoreCipher.init(Cipher.DECRYPT_MODE, key, IvParameterSpec(iv))
        }

        return keystoreCipher
    }

    @Throws(Exception::class)
    private fun getKeyStoreCipherBeforeInit(): Pair<Cipher, SecretKey>? {
        val helper = BiometricsHelper.get()
        val keyStoreCipher = helper.createKeyStoreCipher()
        val keystore = helper.getKeystore()

        val key = keystore.getKey(aliasId, null) as SecretKey?
        val safeKey = key ?: helper.createCipherKeyGenerator().generateKey()

        return Pair(keyStoreCipher, safeKey)
    }

    @Throws(Exception::class)
    fun deleteStoredKey() {
        BiometricsHelper.get().getKeystore().deleteEntry(aliasId)
    }

    /**
     * Check if we can attempt a login via fingerprint
     * @return Boolean true if fingerprint login should be possible
     */
    fun hasAllRequiredStored(): Boolean {
        return try {
            BiometricsHelper.get().getKeystore().containsAlias(aliasId)
                    && keyStoreController.canTryLoginViaBiometric()
        } catch (e: Exception) {
            false
        }
    }
}