/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package dk.nsp.domain.repository

import android.content.Context
import dk.nsp.api.getBackendApiLayer
import dk.nsp.domain.knowledge.ArticleCategory
import dk.nsp.domain.knowledge.SearchResultsForCategory
import dk.nsp.domain.knowledge.toDomainModel
import dk.nsp.shared.ResultWrapper
import dk.nsp.shared.mapSuccess
import dk.nsp.shared.mapSuccessWithTryOrFailed
import dk.nsp.shared.util.asyncDefault
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Deferred
import timber.log.Timber

class ArticlesRepository(context: Context) {

    private val cmsController = context.getBackendApiLayer().cmsController

    fun getArticleSections(scope: CoroutineScope): Deferred<ResultWrapper<List<ArticleCategory>>> =
        scope.asyncDefault {
            val categoriesRes = cmsController.getArticleSections(this).await()

            categoriesRes.mapSuccess {
                it.map { it.toDomainModel() }
            }
        }

    fun searchArticles(
        scope: CoroutineScope,
        query: String
    ): Deferred<ResultWrapper<List<SearchResultsForCategory>>> = scope.asyncDefault {
        val searchResponse = cmsController.searchArticles(this, query).await()

        searchResponse.mapSuccessWithTryOrFailed {
            val categories = it.map { it.toDomainModel() }

            categories.map { category ->
                val articlesThatMatchSearch = category.articles.filter { it.searchOrder != null }

                val articlesThatHasSectionsThatMatchResult =
                    category.articles.filter { it.sections.any { it.searchOrder != null } }

                val articleSectionsThatMatchesSearch =
                    category.articles.flatMap { it.sections.filter { it.searchOrder != null } }

                Timber.d("For category ${category.name}")
                Timber.d("= Article that matches search: ${articlesThatMatchSearch.size} -> ${articlesThatMatchSearch.joinToString { it.title }}")
                Timber.d("== Sections that matches search ${articleSectionsThatMatchesSearch.size} -> ${articleSectionsThatMatchesSearch.joinToString { it.title }}")
                Timber.d("== Articles that has sections that matches search ${articlesThatHasSectionsThatMatchResult.size} -> ${articlesThatHasSectionsThatMatchResult.joinToString { it.title }}")
                SearchResultsForCategory(
                    category,
                    articlesThatMatchSearch,
                    articlesThatHasSectionsThatMatchResult
                )
            }
        }
    }
}