package dk.nsp.util

import android.content.res.Resources
import android.os.Parcelable
import androidx.annotation.PluralsRes
import androidx.annotation.StringRes
import dk.nsp.shared.tryAndLog
import kotlinx.parcelize.Parcelize

sealed class TextModel : Parcelable {
    companion object {
        @JvmStatic
        fun from(@StringRes text: Int): TextResourceModel {
            return TextResourceModel(text)
        }

        @JvmStatic
        fun from(@StringRes text: Int, vararg placeholders: TextModel): TextModel {
            return TextResourcePlaceholderModel(text, placeholders)
        }

        @JvmStatic
        fun from(text: String): TextStringModel {
            return TextStringModel(text)
        }

        /**
         * Used when working with Quantity/Plural string resources
         * @param pluralStringRes Int - The resource ID for the plurals. The value 0 is an invalid identifier.
         * @param quantity Int - The number used to get the correct string for the current language's plural rules
         * @param placeholderValues Array<out String> Only use this parameter if the supplied [pluralStringRes] contains placeholders (see [Resources.getQuantityString] with 3 parameters for more info)
         * @return TextResourceQuantityBindingModel
         */
        fun from(
            @PluralsRes pluralStringRes: Int,
            quantity: Int,
            vararg placeholderValues: TextModel
        ): TextResourceQuantityModel {
            return TextResourceQuantityModel(
                pluralStringRes,
                quantity,
                if (placeholderValues.isNotEmpty()) {
                    placeholderValues
                } else {
                    null
                }
            )
        }
    }

    abstract fun getString(resources: Resources): String?
}

@Parcelize
class TextResourceModel(
    @StringRes val textRes: Int
) : TextModel() {
    override fun getString(resources: Resources): String? =
        tryAndLog { resources.getString(textRes) }
}

@Parcelize
class TextStringModel(
    val text: String
) : TextModel() {
    override fun getString(resources: Resources): String? = text
}

@Parcelize
class TextResourcePlaceholderModel(
    @StringRes val textRes: Int,
    val placeholders: Array<out TextModel>
) : TextModel() {
    override fun getString(resources: Resources): String? = tryAndLog {
        resources.getString(
            textRes,
            *placeholders.toStringArray(resources)
        )
    }
}

@Parcelize
class TextResourceQuantityModel(
    @PluralsRes val pluralStringRes: Int,
    val quantity: Int,
    val placeholderValues: Array<out TextModel>? = null
) : TextModel() {
    override fun getString(resources: Resources): String? = tryAndLog {
        if (placeholderValues == null) {
            resources.getQuantityString(pluralStringRes, quantity)
        } else {
            resources.getQuantityString(
                pluralStringRes,
                quantity,
                *placeholderValues.toStringArray(resources)
            )
        }
    }
}

fun Array<out TextModel>.toStringArray(resources: Resources) = this.map {
    it.getString(resources)
}.toTypedArray()

fun String.toTextModel(): TextModel = TextModel.from(this)
fun String?.toTextModel(fallback: TextModel): TextModel =
    this?.let { TextModel.from(it) } ?: fallback