package dk.nsp.shared


import android.util.Log
import kotlin.contracts.ExperimentalContracts
import kotlin.contracts.InvocationKind
import kotlin.contracts.contract


sealed class ResultWrapper<out T> {
    companion object {
        fun <T, E : Throwable> map(result: T?, error: E): ResultWrapper<T> {
            return if (result != null) {
                Success(result)
            } else {
                Failed(error)
            }
        }
    }
}

// Declaring outside parent to avoid having to reference these by 'ResultWrapper.'
data class Success<out T>(val value: T) : ResultWrapper<T>()
data class Failed(val error: Throwable) : ResultWrapper<Nothing>()


fun <T> T.toSuccess(): Success<T> {
    return Success(this)
}

/**
 * If receiving [ResultWrapper] is a Success, it maps it's [Success.value] with the given mapper function
 * Otherwise it's a [Failed] and it returns this
 * @receiver ResultWrapper<T> The function to be used for mapping of [Success.value]
 * @param mapper Function1<T, R>
 * @return ResultWrapper<R>
 */
fun <T, R> ResultWrapper<T>.mapSuccess(
    mapper: Function1<T, R>
): ResultWrapper<R> = when (this) {
    is Failed -> this
    is Success -> mapper(value).toSuccess()
}

inline fun <T, R> ResultWrapper<T>.mapSuccessWithTryOrFailed(
    mapper: (T) -> R
): ResultWrapper<R> = when (this) {
    is Failed -> this
    is Success -> tryOrFailed {
        mapper(value)
    }
}

fun <T> ResultWrapper<T>.asSuccessValueOrNull(): T? = (this as? Success)?.value

/**
 * catches any Throwable and wraps the result.
 */
@OptIn(ExperimentalContracts::class)
inline fun <T> tryOrFailed(block: () -> T): ResultWrapper<T> {
    contract {
        callsInPlace(block, InvocationKind.EXACTLY_ONCE)
    }
    return try {
        Success(block())
    } catch (e: Throwable) {
        Log.d("tryOrFailed", "failed with", e)
        Failed(e)
    }
}