package dk.nsp.shared.util.sharedprefs

import android.content.Context
import android.content.SharedPreferences
import dk.nsp.shared.tryAndLog

class SharedPrefsHelper(context: Context, settingsName: String = defaultPreferencesName) {

    private val sharedPref = context.getSharedPreferences(settingsName, Context.MODE_PRIVATE)

    //region Biometric
    /**
     * should we use fingerprint ?
     */
    var useBiometrics: Boolean?
        get() = get(usesFingerprintKey, false)
        set(value) = storeOrClear(usesFingerprintKey, value)

    /**
     * whenever we have asked the user to use fingerprint
     */
    var askedForUseBiometrics: Boolean?
        get() = get(askedForUseFingerprint, false)
        set(value) = storeOrClear(askedForUseFingerprint, value)

    fun clearBiometricSettings() {
        useBiometrics = null
        askedForUseBiometrics = null
    }
    //endregion

    //region Common read/write shorthands
    //region Store functions
    fun store(forKey: String, value: String?) = sharedPref.applyWith { putString(forKey, value) }
    fun store(forKey: String, value: Int) = sharedPref.applyWith { putInt(forKey, value) }
    fun store(forKey: String, value: Long) = sharedPref.applyWith { putLong(forKey, value) }
    fun store(forKey: String, value: Boolean) = sharedPref.applyWith { putBoolean(forKey, value) }
    //endregion

    //region Destructive store
    private fun storeOrClear(forKey: String, value: Boolean?) {
        if (value == null) {
            remove(forKey)
        } else {
            store(forKey, value)
        }
    }
    //endregion

    //region Fetch functions
    fun get(forKey: String, default: String? = null) = sharedPref.getString(forKey, default)
    fun get(forKey: String, default: Int) = sharedPref.getInt(forKey, default)
    fun get(forKey: String, default: Long) = sharedPref.getLong(forKey, default)
    fun get(forKey: String, default: Boolean) = sharedPref.getBoolean(forKey, default)
    //endregion

    fun hasValue(forKey: String) = sharedPref.contains(forKey)

    fun remove(forKey: String) = sharedPref.applyWith { remove(forKey) }
    //endregion

    companion object {
        private const val usesFingerprintKey = "UsesFingerprintPreference"
        private const val askedForUseFingerprint = "AskedForUseFingerprintPreference"
        const val defaultPreferencesName = "MinGraviditetAppPrefs"
    }
}


fun Context.getDefaultSharedPrefs() = SharedPrefsHelper(this)

private inline fun SharedPreferences.applyWith(crossinline action: (SharedPreferences.Editor.() -> Unit)) {
    tryAndLog {
        edit().apply(action).apply()
    }
}